/*
    Drools4 Integration Helper
    Copyright (C) 2009  Mathieu Boretti mathieu.boretti@gmail.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.boretti.drools.integration.drools4;

import org.springframework.beans.factory.FactoryBean;

/**
 * This class provide a way to interface with spring.
 * Use
 * <pre>
 * &lt;bean id="XXX"
 * 	class="org.boretti.drools.integration.drools4.DroolsBeanFactory"&gt;
 *		&lt;property name="droolsInterface" value="interface name"/&gt;
 * &lt;/bean&gt;
 * </pre>
 * To use it. The passed interface must have been annotated with the
 * right annotation.
 * @author mbo
 *
 */
public class DroolsBeanFactory implements FactoryBean {

	private Class<?> droolsInterface;
	
	@Override
	public Object getObject() throws Exception {
		return new DroolsProvider().getService(droolsInterface);
	}

	@Override
	public Class<?> getObjectType() {
		return droolsInterface;
	}

	@Override
	public boolean isSingleton() {
		return false;
	}

	public void setDroolsInterface(Class<?> droolsInterface) {
		this.droolsInterface = droolsInterface;
	}


}
