/*
    Drools4 Integration Helper
    Copyright (C) 2009  Mathieu Boretti mathieu.boretti@gmail.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.boretti.drools.integration.drools4.annotations;

import java.lang.annotation.Retention;
import java.lang.annotation.Target;

import org.boretti.drools.integration.drools4.DroolsSessionType;

import static java.lang.annotation.RetentionPolicy.RUNTIME;
import static java.lang.annotation.ElementType.METHOD;

/**
 * This annotations marks a method of an interface that map to drools rules.
 * 
 * Usage of this annotation is optional. If omitted, session is assumed as NONE.
 * 
 * @author mbo
 *
 */
@Retention(RUNTIME)
@Target(METHOD)
public @interface DroolsMethod {
	/**
	 * This is the session type.
	 * <ul>
	 * <li>NONE=purely stateless</li>
	 * <li>NEW=every time create a new session</li>
	 * <li>END=every time delete the existing session at the end</li>
	 * <li>JOIN=use existing session or if no session execute stateless</li>
	 * <li>REQUIRE=must be used from within a session</ul>
	 * </ul>
	 * @return
	 */
	DroolsSessionType session() default DroolsSessionType.NONE;
}
