/*
    Drools4 Integration Helper
    Copyright (C) 2009  Mathieu Boretti mathieu.boretti@gmail.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.boretti.drools.integration.drools4.implementation;

import java.lang.annotation.Annotation;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;
import org.boretti.drools.integration.drools4.DroolsInterface;
import org.boretti.drools.integration.drools4.DroolsParameterHandling;
import org.boretti.drools.integration.drools4.DroolsSessionType;
import org.boretti.drools.integration.drools4.annotations.DroolsMethod;
import org.boretti.drools.integration.drools4.annotations.DroolsParameter;


/**
 * @author mbo
 * 
 */
class DroolsProxy<T> implements InvocationHandler {
	
	private final Logger logger = Logger.getLogger(this.getClass());
			
	private List<Object> produceFact(Method m,Object[] args) {
		List<Object> fact = new ArrayList<Object>();
		if (args==null) return fact;
		if (args.length==0) return fact;
		Annotation[][] ass = m.getParameterAnnotations();
		for(int i=0;i<args.length;i++) {
			Annotation[] as = ass[i];
			DroolsParameterHandling handling = DroolsParameterHandling.NONE;
			for(Annotation a:as) {
				if (a.annotationType().equals(DroolsParameter.class)) {
					handling = ((DroolsParameter)a).handling();
					break;
				}
			}
			if (handling==null) handling = DroolsParameterHandling.NONE;
			if (DroolsParameterHandling.NONE.equals(handling)) {
				fact.add(args[i]);
			} else if (DroolsParameterHandling.IGNORED.equals(handling)) {
			} else if (DroolsParameterHandling.TOFLAT1.equals(handling)) {
				if (args[i] instanceof Object[]) for(Object o:(Object[])args[i]) fact.add(o);
				else if (args[i] instanceof Collection) for(Object o:(Collection<?>)args[i]) fact.add(o);
				else fact.add(args[i]);
			} else if (DroolsParameterHandling.TOFLATALL.equals(handling)) {
				addFlat(fact,args[i]);
			} 
		}
		return fact;
	}
	
	private void addFlat(List<Object> fact,Object args) {
		if (args instanceof Object[]) for(Object o:(Object[])args) addFlat(fact,o);
		else if (args instanceof Collection) for(Object o:(Collection<?>)args) addFlat(fact,o);
		else fact.add(args);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.reflect.InvocationHandler#invoke(java.lang.Object,
	 *      java.lang.reflect.Method, java.lang.Object[])
	 */
	@Override
	public Object invoke(Object proxy, Method method, Object[] args)
			throws Throwable {
		if (method.getDeclaringClass().equals(clazz)) {
			DroolsMethod annotation = null;
			DroolsSessionType session = DroolsSessionType.NONE;
			try {
				annotation = method.getAnnotation(DroolsMethod.class);
			} catch (NullPointerException ex) {}
			List<Object> fact = produceFact(method,args);
			Class<?> c = method.getReturnType();
			if (annotation!=null && annotation.session()!=null) session = annotation.session();
			Iterator<?> result=null;
			if (DroolsSessionType.NONE.equals(session)) {
				result = inter.runStatelessSession(fact);
			} else if (DroolsSessionType.NEW.equals(session)) {
				if (inter.getCurrentSession()!=null) {
					inter.getCurrentSession().halt();
					inter.setCurrentSession(null);
				}
				result=inter.runStatefulSession(fact, false);
			} else if (DroolsSessionType.END.equals(session)) {
				result=inter.runStatefulSession(fact, true);
				inter.getCurrentSession().halt();
				inter.setCurrentSession(null);
			} else if (DroolsSessionType.JOIN.equals(session)) {
				if (inter.getCurrentSession()==null) result = inter.runStatelessSession(fact);
				else result = inter.runStatefulSession(fact, true);
			} else if (DroolsSessionType.REQUIRE.equals(session)) {
				result = inter.runStatefulSession(fact, true);
			}
			return inter.getResult(c, result);
		} else if (method.getDeclaringClass().equals(DroolsInterface.class)) {
			return method.invoke(inter, args);
		}
		String msg = "Unable to handle the received parameter";
		logger.error(msg);
		throw new IllegalArgumentException(msg);
	}
	
	private Class<T> clazz;
	
	private DroolsInterfaceImplementation<T> inter;
	
	public DroolsProxy(Class<T> clazz,DroolsInterfaceImplementation<T> inter) {
		this.clazz=clazz;
		this.inter=inter;
	}

}
