/*
    Drools4 Integration Helper
    Copyright (C) 2009  Mathieu Boretti mathieu.boretti@gmail.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.boretti.drools.integration.drools4.implementation;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import org.apache.log4j.Logger;
import org.boretti.drools.integration.drools4.exceptions.ClassCastDroolsError;
import org.boretti.drools.integration.drools4.exceptions.CompilationDroolsError;
import org.boretti.drools.integration.drools4.exceptions.DroolsError;
import org.boretti.drools.integration.drools4.exceptions.IODroolsError;
import org.drools.RuleBase;
import org.drools.RuleBaseFactory;
import org.drools.compiler.PackageBuilder;
import org.drools.compiler.PackageBuilderConfiguration;

/**
 * @author mbo
 * 
 */
class DroolsSourceInterfaceImplementation<T> extends
		DroolsInterfaceImplementation<T> {
	
	private static final Logger logger = Logger.getLogger(DroolsSourceInterfaceImplementation.class);

	public DroolsSourceInterfaceImplementation(Class<T> clazz,
			String resourceName) {
		super(clazz, generateRuleBase(clazz, resourceName));
	}

	private static final RuleBase generateRuleBase(Class<?> clazz,String resourceName) {
		RuleBase ruleBase = RuleBaseFactory.newRuleBase();
		try {
			InputStream is = clazz.getResourceAsStream(resourceName);
			if (is==null) {
				String msg = "IO Exception, resource not found";
				logger.error(msg);
				throw new IODroolsError(msg);
			}
			InputStreamReader isr = new InputStreamReader(is);
			PackageBuilderConfiguration conf = new PackageBuilderConfiguration();
			conf.setClassLoader(clazz.getClassLoader());
			PackageBuilder builder = new PackageBuilder(conf);
			builder.addPackageFromDrl(isr);
			if (builder.getErrors()!=null) {
				if (builder.getErrors().getErrors()!=null) {
					if (builder.getErrors().getErrors().length>0) {
						StringBuffer sp = new StringBuffer();
						for(org.drools.compiler.DroolsError er :builder.getErrors().getErrors()) {
							logger.error("Drools compilation error :"+er.getMessage());
							sp.append(er.getMessage()+";\n");
						}
						String msg = "Invalid Drools :"+sp.toString();
						logger.error(msg);
						throw new CompilationDroolsError(msg);
					}
				}
			}
	   		isr.close();
	   		ruleBase.addPackage(builder.getPackage());
		} catch (IOException e) {
			String msg = "IO Exception "+e.getMessage();
			logger.error(msg);
			throw new IODroolsError(msg,e);
		} catch (ClassNotFoundException e) {
			String msg = "Class Cast Exception "+e.getMessage();
			logger.error(msg);
			throw new ClassCastDroolsError(msg,e);
		} catch (Exception e) {
			String msg = "Exception "+e.getMessage();
			logger.error(msg);
			throw new DroolsError(msg,e);
		}
		return ruleBase;
	}

}
